//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/
import javax.swing.*; 
import java.awt.*;
import java.awt.event.*; 

/**
 * 
 * <p>
 * 
 * @author Aaron Cooper
 * @version 1.0 8-Jul-2005
 * 
 * A Dialog message window that is presented when rare errors occur (such as IO write/read failures)
 * In theory, user should never see this window.  If not, an email is shown and the user is encouraged 
 * to copy the message text and sent it to the email address. 
 */
public class BugErrorDialog extends JDialog implements ActionListener {

	protected static final int EMPTY_CMP_SIZE = 20; 
	
	// location of image for dialog's icon
	protected static final String IMG_PATH = "images/embarassed.jpg"; 
	
	// email address where bug reports should be sent
	protected static final String SUPPORT_EMAIL = "ccampeanu@upei.ca"; 
	
	private JLabel msgLabel; 
	private JPanel mainPanel, picturePanel;
	private JButton closeButton; 
	
	private JScrollPane msgScrPane; 	
	private JTextArea msgTextArea; 
	
	private JLabel picture; 

	// use when you want to prepend a message before the stacktrace 
	public BugErrorDialog() {
		// create and setup the components
		createComponents();
		setupComponents(); 
		addComponents(); 
	}

	public BugErrorDialog(Exception e) {
		// create and setup the components
		createComponents();
		setupComponents(); 
		addComponents(); 
		
		// set the error message to display
		this.setMessage(e);
	}
	
	protected void createComponents() {
		// use HTML to make a multi-line label: see http://java.sun.com/docs/books/tutorial/uiswing/components/html.html
		msgLabel = new JLabel("<html>Oops!  A fault in JGrail has just occurred, likely due to poor programming.  <br> " +
											"If the message below is not useful, please copy it and email it to " + SUPPORT_EMAIL + 
											" , so that it can be investigated <br><br>" +
											"Hopefully, this hasn't caused too much inconvenience.");
 
		msgLabel.setHorizontalAlignment(SwingConstants.CENTER);
		
		// create the "oops" icon
		picture		  = new JLabel(new ImageIcon(IMG_PATH) );
		
		picturePanel = new JPanel(); 
		mainPanel = new JPanel();
				
		closeButton 	 = new JButton("Close");
	
		msgTextArea = new JTextArea(); 
		msgScrPane  = new JScrollPane(msgTextArea);  	
	}
	
	protected void setupComponents() {
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS) );
		picturePanel.setLayout(new BoxLayout(picturePanel, BoxLayout.X_AXIS) );
		
		// ensure all components are center-aligned
		msgLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		msgScrPane.setAlignmentX(Component.CENTER_ALIGNMENT);
		picturePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		msgTextArea.setLineWrap(true);

		// register for button listeners		
		closeButton.addActionListener(this);
		
		// set the dialog size
		this.setSize(650, 350);
		this.setTitle("Oops...a fault has occured");
		this.setModal(true);
	}
	
	protected void addComponents() {
		picturePanel.add(Box.createRigidArea(new Dimension(EMPTY_CMP_SIZE / 2, 0) ));
		picturePanel.add(picture);
		picturePanel.add(Box.createRigidArea(new Dimension(EMPTY_CMP_SIZE, 0) ));
		picturePanel.add(msgScrPane);
		picturePanel.add(Box.createRigidArea(new Dimension(EMPTY_CMP_SIZE / 2, 0) ));

		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		mainPanel.add(msgLabel);
		
		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		mainPanel.add(picturePanel); 

		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		mainPanel.add(closeButton);
		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		
		this.setContentPane(mainPanel);
	}

	// close the dialog when the user clicks the Close button	
	public void actionPerformed(ActionEvent ae) {
		this.dispose();	
	}
	

	// prints the stack trace to the text area
	// (i.e. in essence, output from e.printStackTrace() are placed on the GUI widget)
	public void setMessage(Exception e) {
		if ( e != null ) {
			// get the array of stack trace elements
			StackTraceElement[] stackTrace = e.getStackTrace();
			
			for (int i=0; i < stackTrace.length; i++) {
				// insert the stack trace element
				msgTextArea.append(stackTrace[i].toString() ); 
	
				// insert a new line
				msgTextArea.append("\n");
			}
		}			
	}

	// adds text to the message area	
	public void addText(String text) {
		msgTextArea.append(text);
	}

	public static void main(String[] args) {
		BugErrorDialog d = new BugErrorDialog();
		d.show();
	}
}
