import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.plaf.metal.*;
import javax.swing.plaf.basic.*;
import java.io.*;

/**
* SteppedComboBox represents a JComboBox whose
* "drop-down popup box" can be larger than the width
* of the combobox itself.
* <p>
* This allows the combo box to possess a static width,
* while its "drop-down popup box" can show all values, including
* those whose width is larger than the combo box.
* <p>
* For more information, see the following:
* http://www.codeguru.com/java/articles/163.shtml
* <p>
* Bug Note: When the window that contains the combo is shrunk,
*           the combo and its popup area get smaller....subsequently, the
*           popup area does not grow larger as desired.  Investigate later...
*
* @author Nobuo Tamemasa
* @author Aaron Cooper (added comments and little touch-ups)
* @version 1.0 12/12/98
*/

// Aaron Cooper -- I only commented the sections I modified //

class MyComboBoxPopup extends BasicComboPopup {
  public MyComboBoxPopup(JComboBox comboBox) {
    super(comboBox);
  }
  
  protected JScrollPane createScroller() {
    return new JScrollPane(list,
     ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
     ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED );
  }
}
 
class SteppedComboBoxUI extends BasicComboBoxUI {
  /*protected ComboPopup createPopup() {
    return new MyComboBoxPopup(comboBox);
  }
  */
  protected ComboPopup createPopup() {
    BasicComboPopup popup = new BasicComboPopup( comboBox ) {

      public void show() {
				JList cbList = getList(); 
				
        Dimension popupSize = ((SteppedComboBox)comboBox).getPopupSize();

        // Aaron Cooper May 26, 2004
        // if the popup box is smaller than the combo width, make it the same width
        if ( popupSize.width < comboBox.getWidth() ) {
          popupSize.width = comboBox.getWidth();
        }
          
        popupSize.setSize( popupSize.width,
          getPopupHeightForRowCount( comboBox.getMaximumRowCount() ) );

        Rectangle popupBounds = computePopupBounds( 0,
          comboBox.getBounds().height, popupSize.width, popupSize.height);
          
        scroller.setMaximumSize( popupBounds.getSize() );
        scroller.setPreferredSize( popupBounds.getSize() );
        scroller.setMinimumSize( popupBounds.getSize() );
        cbList.invalidate();            
        int selectedIndex = comboBox.getSelectedIndex();
        if ( selectedIndex == -1 ) {
          cbList.clearSelection();
        } else {
          cbList.setSelectedIndex( selectedIndex );
        }            
        cbList.ensureIndexIsVisible( list.getSelectedIndex() );

          // Aaron Cooper May 2004
          // commented out... don't think it's necessary
          //setLightWeightPopupEnabled( comboBox.isLightWeightPopupEnabled() );

				// from superclass JPopupMenu	
        show( comboBox, popupBounds.x, popupBounds.y );
      }
    };
		
    popup.getAccessibleContext().setAccessibleParent(comboBox);
    return popup;
  }
}

public class SteppedComboBox extends JComboBox {
  protected int popupWidth;
  
  public SteppedComboBox(ComboBoxModel aModel) {
    super(aModel);
    setUI(new SteppedComboBoxUI());
    popupWidth = 0;
  }

  public SteppedComboBox(final Object[] items) {
    super(items);
    setUI(new SteppedComboBoxUI());
    popupWidth = 0;
  }

  public SteppedComboBox(final Object[] items, int maxWidth, int maxHeight) {
    this(items);
    setSize(maxWidth, maxHeight); 
  }

  public SteppedComboBox(int maxWidth, int maxHeight) {
    super();
    setUI(new SteppedComboBoxUI() );
    setSize(maxWidth, maxHeight);
  }

  public SteppedComboBox(Vector items) {
    super(items);
    setUI(new SteppedComboBoxUI());
    popupWidth = 0;
  }

  /**
  * Sets the combo's max width, and it's height to a fixed amount
  *
  * @author Aaron Cooper May 2004
  */ 
	public void setSize(int maxWidth, int staticHeight) {
    // set the sizes
    Dimension d = this.getPreferredSize();
    this.setPreferredSize(new Dimension(maxWidth, staticHeight) );
    this.setMaximumSize(new Dimension(maxWidth, staticHeight) );
    this.setMinimumSize(new Dimension(this.getMinimumSize().width, staticHeight) );
      
    this.setPopupWidth(d.width); 

    // create a border around the combo box		
    setBorder(BorderFactory.createEtchedBorder() ); 
	}
  
  
  public void setPopupWidth(int width) {
    popupWidth = width;
  }
  
  public Dimension getPopupSize() {
    Dimension size = getSize();
    if (popupWidth < 1) popupWidth = size.width;
    return new Dimension(popupWidth, size.height);
  }
}
