//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.*; 
import java.awt.*;
import java.awt.event.*;

import java.io.*;  
/**
 * 
 * <p>
 * 
 * @author Aaron Cooper
 * @version 1.0 2-Jul-2005
 * 
 * This dialog window is presented to the user if the program has problems parsing the conf file
 */
public class ConfFileParseErrorDialog extends JDialog implements ActionListener {

	protected static final int EMPTY_CMP_SIZE = 2; 
	protected static final String IMG_PATH = "images/setup.jpg"; 

	public static final int CREATE_BTN    = 1; 
	public static final int RESTORE_BTN = 2;
	public static final int MANUAL_BTN  = 3;
	

	private JLabel msgLabel, optionLabel; 
	private JPanel buttonPanel, mainPanel;
	private JButton createButton, restoreButton, manualButton; 
	
	private JScrollPane msgScrPane; 	
	private JTextArea msgTextArea; 
	private JLabel picture;

	private int clickedButton;

	public ConfFileParseErrorDialog(String message) {
		// create and setup the components
		createComponents();
		setupComponents(); 
		addComponents(); 
		
		// set the error message to display
		this.setMessage(message);

		// set the dialog size
		this.setSize(500, 400);
		
		this.setTitle("JGrail's Configuration File");
		
		this.setModal(true);
	}
	
	protected void createComponents() {
		// use HTML to make a multi-line label: see http://java.sun.com/docs/books/tutorial/uiswing/components/html.html

		msgLabel = new JLabel("<html>Hi. I had some trouble reading JGrail's configuration file. <br><br><br> This is most likely because this is the first time you are running JGrail and there is no configuration file in your home directory. If, indeed, this is the first time you are running JGrail, you just have to create a default configuration file. ");

		picture		  = new JLabel(new ImageIcon(IMG_PATH) );

		msgLabel.setHorizontalAlignment(SwingConstants.CENTER);
		
		optionLabel = new JLabel("How would you like to handle this?");
		
		buttonPanel = new JPanel();
		mainPanel = new JPanel();
				
		createButton 	 = new JButton("Create Default Configuration File"); 
		restoreButton = new JButton("Restore Backup Configuration File");
		manualButton = new JButton("Manually Investigate");
	
		msgTextArea = new JTextArea(); 
		msgScrPane  = new JScrollPane(msgTextArea);  	
	}
	
	protected void setupComponents() {
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS) );
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.Y_AXIS) );
		
		// ensure all components are center-aligned
		picture.setAlignmentX(Component.CENTER_ALIGNMENT);
		msgLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		msgScrPane.setAlignmentX(Component.CENTER_ALIGNMENT);
		optionLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		buttonPanel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		
		msgTextArea.setLineWrap(true);

		// register for button listeners		
		createButton.addActionListener(this);
		restoreButton.addActionListener(this);
		manualButton.addActionListener(this);
		
		// register a new event for handling closing via the "X" button
		// act as if the user clicked the manual button
		this.addWindowListener(new WindowAdapter() {
		  public void windowClosing(WindowEvent we) {
			dispose(MANUAL_BTN);
		  }
		});
	}
	
	protected void addComponents() {
		buttonPanel.add(createButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		buttonPanel.add(restoreButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		buttonPanel.add(manualButton);

		mainPanel.add(picture);
		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		mainPanel.add(msgLabel);
		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		mainPanel.add(msgScrPane);
		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		mainPanel.add(optionLabel);
		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
		mainPanel.add(buttonPanel);
		mainPanel.add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));

		this.setContentPane(mainPanel);
	}

	
	// handles the event when user clicks the Create button
	private void handleCreateButton() throws Exception {
		Setup s = new Setup(); 
		
		// if setup directory already exists, then copy the config file
		if ( !s.create() ) {
			System.out.println("going to copy over the default config file...");
				
			// copy the default config file
			s.copyDfltConfFile(); 
				
			// show OK message
			JOptionPane.showMessageDialog(this, "Successfully created a default configuration file.  JGrail will now start");
				
			// dispose of the dialog
			this.dispose(CREATE_BTN); 
		}
	}
	
	//	handles the event when user clicks the Restore button
	protected void handleRestoreButton() throws Exception {
		Setup s = new Setup(); 
	
		// restore the backup version of the conf file
		s.restoreBackup(); 

		// show OK message
		JOptionPane.showMessageDialog(this, "Successfully restored the backup version of the configuration file.  JGrail will now start");
		
		// dispose of the dialog
		this.dispose(RESTORE_BTN); 
	}
	
	//	handles the event when user clicks the Manual button
	protected void handleManualButton() {
		// build the path to the configuration file
		String confFile = ConfFile.CONF_FULL_PATH; 
				
		// show OK message
		JOptionPane.showMessageDialog(this, "You have decided to manually investigate the configuration file, which is located at " + confFile + ". JGrail will now close"); 
	
		// dispose of the dialog
		this.dispose(MANUAL_BTN); 
	}
	
	public void actionPerformed(ActionEvent ae) {
		JButton source = (JButton) ae.getSource(); 

		try {		
			// user wants to create a new default conf file
			if ( source == createButton ) {
				handleCreateButton(); 
			}
			
			// user wants to restore the backup version of the conf file
			else if ( source == restoreButton ) {
				handleRestoreButton(); 
			}
			
			// user has decided to manually investigate the problem with the conf file
			else if ( source == manualButton ) {
				handleManualButton();
			}
			
			// this case shouldn't happen.. don't do anything
			else {
				return;
			}
		}	
		
		// show "Bug Dialog" here....
		catch (Exception e) {
			Global.showBugErrorDialog(e);
			
			// dispose of the dialog.. and MANUAL_BTN will cause JGrailGui to halt the program entirely
			this.dispose(MANUAL_BTN);
		}
	}

	// shows the dialog and returns the button that was clicked
	public int showDlg() {
		// only enable the restore button if the backup version of the conf file exists
		if ( !Global.backupFileExists() )
			this.restoreButton.setEnabled(false);

		// now show the dialog 
		super.show(); 
		
		// and return the button that was clicked by the user
		return getClickedButton(); 	
	}
	
	// set the clicked button before disposing
	public void dispose(int clickedButton) {
		setClickedButton(clickedButton);
		super.dispose(); 
	}
	
	protected void setClickedButton(int button) { this.clickedButton = button; }
	protected int getClickedButton() { return this.clickedButton; }
	
	public void setMessage(String message) {
		msgTextArea.setText(message); 	
	}

	public static void main(String[] args) {
		ConfFileParseErrorDialog d = new ConfFileParseErrorDialog("test");
		d.show();
	}
}
