//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*; 
import javax.swing.*; 

import java.util.Vector;

public class LeftPanel extends AbstractPanel {

  protected static final int MAX_WIDTH        = 400;
  protected static final int COMBO_MAX_WIDTH  = MAX_WIDTH - 20;
  protected static final int COMBO_MAX_HEIGHT = 22;

  private ListFuncsPanel  listFuncsPanel;
  private FuncsPanel 		  funcsPanel;
  private UnaryFuncsPanel unaryFuncsPanel; 

  public LeftPanel(JCheckBox hideFirstChkBox, JCheckBox hideSecondChkBox, Function[] unaryFuncs, Function[] binaryFuncs) {
    createComponents(hideFirstChkBox, hideSecondChkBox, unaryFuncs, binaryFuncs);
    addComponents();
  }

  //// GETTERS & SETTERS /////
  public FuncsPanel getFuncsPanel()           { return this.funcsPanel; }
  protected void setFuncsPanel(FuncsPanel funcsPanel) { this.funcsPanel = funcsPanel; }
  
  public UnaryFuncsPanel getUnaryFuncsPanel() { return this.unaryFuncsPanel; }
  protected void setUnaryFuncsPanel(UnaryFuncsPanel unaryFuncsPanel) { this.unaryFuncsPanel = unaryFuncsPanel; }
  
  public ListFuncsPanel getListFuncsPanel()   { return this.listFuncsPanel; }
  protected void setListFuncsPanel(ListFuncsPanel listFuncsPanel) { this.listFuncsPanel = listFuncsPanel; }

  
  public void changeListData(Function[] unaryFuncs, Function[] binaryFuncs) {
    getFuncsPanel().changeListData(unaryFuncs, binaryFuncs);
    getUnaryFuncsPanel().changeListData(unaryFuncs);
  }

  // required abstract methods
	protected void setupComponents() {}
  protected void createComponents() {}

	protected void createComponents(JCheckBox hideFirstChkBox, JCheckBox hideSecondChkBox,
                                  Function[] unaryFuncs, Function[] binaryFuncs) {
    // components added to left
    setListFuncsPanel(new ListFuncsPanel() );
    setFuncsPanel(new FuncsPanel(COMBO_MAX_WIDTH, COMBO_MAX_HEIGHT, hideFirstChkBox, hideSecondChkBox, unaryFuncs, binaryFuncs) );
    setUnaryFuncsPanel(new UnaryFuncsPanel(COMBO_MAX_WIDTH, COMBO_MAX_HEIGHT, getListFuncsPanel().getFuncList(), unaryFuncs) );
	}

	protected void addComponents() {
    GridBagConstraints c = new GridBagConstraints();
    setLayout(new GridBagLayout() );

    c.anchor = GridBagConstraints.CENTER;
      //c.anchor  = GridBagConstraints.FIRST_LINE_START;
    c.fill    = GridBagConstraints.BOTH;

    c.gridx = 0;
    c.gridy = 0;

    c.weightx = 1.0;
    c.weighty = 0.0;
    c.insets = new Insets(10, 10, 10, 10);

    // left panel
    add(getFuncsPanel(), c);
      //add(Box.createRigidArea(new Dimension(0, 10) ));

    c.gridx = 0;
    c.gridy = 1;

    c.weightx = 1.0;
    c.weighty = 0.0;
    c.insets = new Insets(10, 10, 10, 10);

    add(getUnaryFuncsPanel(), c);
      //add(Box.createRigidArea(new Dimension(0, 10) ));

    c.gridx = 0;
    c.gridy = 2;

    c.weightx = 1.0;
    c.weighty = 1.0;
    c.insets = new Insets(10, 10, 10, 10);

    add(getListFuncsPanel(), c);

    // set the sizes
    setMaximumSize(new Dimension(MAX_WIDTH, getMaximumSize().height) );
    setPreferredSize(getMaximumSize() );
    setMinimumSize(new Dimension(getMaximumSize().width/2, getMaximumSize().height/2) );
	}

}
