//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*; 

class ListFuncsPanel extends AbstractPanel implements ActionListener {

  protected static final int EMPTY_CMP_SIZE    = 5;
  protected static final int EMPTY_BDR_SIZE    = 5;

  protected static final String PANEL_TITLE    = "List of Functions to Perform";
  protected static final String BTN_TITLE      = "Remove";

  private JScrollPane funcScrPane; 
  private JButton     removeButton;

  public ListFuncsPanel() {
    super(); 

    setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS) );
    setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createTitledBorder(PANEL_TITLE),
      BorderFactory.createEmptyBorder(EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE) )
    );

    createComponents();
    setupComponents();
    addComponents();
  }

  // GETTERS & SETTERS //
  protected JScrollPane getFuncScrPane() { return this.funcScrPane; }
  protected void setFuncScrPane(JScrollPane funcScrPane) { this.funcScrPane = funcScrPane; }

  protected JButton getRemoveButton() { return this.removeButton; }
  protected void setRemoveButton(JButton removeButton) { this.removeButton = removeButton; }

  public JList getFuncList() { return (JList) getFuncScrPane().getViewport().getView(); }
    
  
  protected void createComponents() {
    DefaultListModel model = new DefaultListModel();

    setFuncScrPane(new JScrollPane(new JList(model) ));
    setRemoveButton(new JButton(BTN_TITLE) );
  }

  protected void setupComponents() {
    getFuncScrPane().setAlignmentX(CENTER_ALIGNMENT);
    getRemoveButton().setAlignmentX(CENTER_ALIGNMENT);
    getRemoveButton().addActionListener(this);
  }

  protected void addComponents() {
    add(getFuncScrPane() );
    add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
    add(getRemoveButton() );
  }


  public void actionPerformed(ActionEvent ae) {
    JList list             = getFuncList();
    DefaultListModel model = (DefaultListModel) list.getModel();
      //ListSelectionModel selectionModel = list.getSelectionModel();

    if ( !list.isSelectionEmpty() ) {
      int[] selected = list.getSelectedIndices();

      // remove items, starting from the bottom and going up
      for (int i=selected.length-1; i >= 0; i--) { 
        model.removeElementAt(selected[i]);
      }
    }
  }

}
