//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import javax.swing.table.*;
import java.util.Vector;
import java.util.Enumeration;


public class ConfTablePanel extends AbstractPanel implements ActionListener, TableModelListener {

  //// CONSTANTS
  protected static final String FIRST_COL_NAME  	  = "Name";
  protected static final String SECOND_COL_NAME = "Description";
  protected static final String THIRD_COL_NAME     = "String Argument";  
  
  protected static final String ADD_AFTER_BTN_TITLE    = "Add Row After";
  protected static final String ADD_BEFORE_BTN_TITLE = "Add Row Before";
  protected static final String DELETE_BTN_TITLE     	 = "Delete Row";    

  protected static final int NUM_COLS   = 3;

  //// INSTANCE VARIABLES
  private JButton addBeforeButton, addAfterButton, deleteButton; 
  private JPanel  buttonPanel;
  private boolean isModified; 

  private DefaultTableModel tableModel;   
  private JTable            funcTable;
  private JScrollPane       tableScrPane;

  
	public ConfTablePanel(int tableWidth, int tableHeight, Function[] arrData) {	
    createComponents(arrData);
    setupComponents(tableWidth, tableHeight);

    setLayout(new GridBagLayout() );
    setModified(false);
    
    addComponents();
	}

  ///// EVENTS ///////
  public void actionPerformed(ActionEvent ae) {
    JButton source      = (JButton) ae.getSource();
    Object[] empty      = new Object[NUM_COLS];

    int nextSelectedRow = -1;
    int currSelectedRow = getFuncTable().getSelectedRow();

    // handle DELETE
    if ( source == getDeleteButton() ) {
      if ( currSelectedRow != Global.NOT_FOUND ) {
          //int selectedCol = tblFuncs.getSelectedColumn();
        getTableModel().removeRow(currSelectedRow);

		// if we deleted the last row, then move the selected row up by one        	
        if ( currSelectedRow >= getFuncTable().getRowCount() )
			nextSelectedRow = currSelectedRow - 1;
		// otherwise the rows will move up by one automatically, so retain the selection at the current row
		else			
        	nextSelectedRow = currSelectedRow;

        // place focus on the first row, if we go beyond it
        if ( nextSelectedRow < 0 )
          nextSelectedRow = 0;
      }
    }

    // handle insert BEFORE
    else if ( source == getAddBeforeButton() ) {
      //  add row to the end of table, if none selected
      if ( currSelectedRow == Global.NOT_FOUND ) {
        currSelectedRow = getTableModel().getRowCount();
      }
      getTableModel().insertRow(currSelectedRow, empty);

      // keep focus on the same row
      nextSelectedRow = currSelectedRow;
    }

    // handle insert AFTER
    else {
      //  add row to the end of table, if none selected      
      if ( currSelectedRow == Global.NOT_FOUND ) {
        currSelectedRow = getTableModel().getRowCount() - 1;
      }
      getTableModel().insertRow(currSelectedRow + 1, empty);

      // focus will be on the row which is one below current selection
      nextSelectedRow = currSelectedRow + 1;
    }

    // move the selection and focus to the newly created/deleted cell
    if ( getTableModel().getRowCount() > 0 ) {
      setSelectedCell(nextSelectedRow, this.getFuncTable().getSelectedColumn() );
    }
  }

  public void tableChanged(TableModelEvent e) {
    setModified(true);
  }

  // required abstract methods....
  protected void createComponents() {}
  protected void setupComponents() {}

	protected void createComponents(Function[] functions) {
    	setAddBeforeButton(new JButton(ADD_BEFORE_BTN_TITLE) );
    	setAddAfterButton(new JButton(ADD_AFTER_BTN_TITLE) );
    	setDeleteButton(new JButton(DELETE_BTN_TITLE) );
    	setButtonPanel(new JPanel() );

    	createTable(functions);
    }

  protected void setupComponents(int tableWidth, int tableHeight) {
    setupButtons();
    setupTable(tableWidth, tableHeight);
  }

  protected void addComponents() {
    setupGridBagLayout();
  }

  private void setupGridBagLayout() {
    GridBagConstraints c = new GridBagConstraints();

    // TABLE on top //
    c.gridx = 0;
    c.gridy = 0;

    c.weightx = 1.0;
    c.weighty = 0.6;

	c.gridwidth  = GridBagConstraints.REMAINDER;
	c.fill 			  = GridBagConstraints.BOTH;
	
    add(getTableScrPane(), c);
    
	// BUTTONS on the bottom //
	c.gridx = 0;
	c.gridy = 1;

	c.weightx = 1.0;
	c.weighty = 0.4;

	c.anchor	 = GridBagConstraints.CENTER;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill 			 = GridBagConstraints.NONE;

	add(getButtonPanel(), c);
	
	this.setBackground(getButtonPanel().getBackground() ); 	
  }

  private void createTable(Function[] functions) {
    String[] colNames = {FIRST_COL_NAME, SECOND_COL_NAME, THIRD_COL_NAME};
    int firstColWidth = 0;

    if ( functions != null )
      setTableModel(new CustomTableModel(colNames, functions.length) );
    else
      setTableModel(new CustomTableModel(colNames, 0) );

    if ( functions != null ) {
      for (int i=0; i < functions.length; i++) {
        if ( functions[i] != null ) {
          getTableModel().setValueAt(functions[i].getName(),  i, Global.FIRST_COL);
          getTableModel().setValueAt(functions[i].getDesc(), i, Global.SECOND_COL);
          getTableModel().setValueAt(new Boolean(functions[i].takesStringArg() ), i, Global.THIRD_COL);
        }
      }
    }

    setFuncTable(new JTable(getTableModel() ));
    setTableScrPane(new JScrollPane(getFuncTable() ));
  }


  

  protected void setupButtons() {
    getButtonPanel().setLayout(new BoxLayout(getButtonPanel(), BoxLayout.LINE_AXIS) );

    // make buttons equal size
    setButtonsEqualSized();

    // register button clicking events
    getAddAfterButton().addActionListener(this);
    getAddBeforeButton().addActionListener(this);
    getDeleteButton().addActionListener(this);

    getButtonPanel().add(getAddBeforeButton() );
    getButtonPanel().add(Box.createRigidArea(new Dimension(15, 0) ));
    getButtonPanel().add(getAddAfterButton() );
	getButtonPanel().add(Box.createRigidArea(new Dimension(15, 0) ));    
    getButtonPanel().add(getDeleteButton() );
  }

  	
  // ensures that the second colum (description) is larger than the other two 	
  protected void setInitialColWidths() { 
	TableColumn column = null;
	
	for (int i = 0; i < NUM_COLS; i++) {
		column = this.getFuncTable().getColumnModel().getColumn(i);
		
		if ( i == Global.SECOND_COL)
			column.setPreferredWidth(250);
		else 
			column.setPreferredWidth(30);
	}
  }

  protected void setupTable(int tableWidth, int tableHeight) {
    getFuncTable().setRowSelectionAllowed(false);

    // set default view size
    getFuncTable().setPreferredScrollableViewportSize(new Dimension(tableWidth, tableHeight) );

    // register event handling
    getTableModel().addTableModelListener(this);
    
    // setup initial column widths  
    setInitialColWidths();
    
	// register a mouse listener
    getFuncTable().addMouseListener(new MouseAdapter() { 
		// used with validation function
		// once user clicks on a cell, disable row selection, and thus the highlighting of the selected row
		public void mouseClicked(MouseEvent e) {
			if ( getFuncTable().getRowSelectionAllowed() ) {
				getFuncTable().setRowSelectionAllowed(false);				
			}
		}
    });

  }


	
	/* Must implement a custom table model so that the last column appears as a check box */ 
	/* http://java.sun.com/docs/books/tutorial/uiswing/components/table.html */
	class CustomTableModel extends DefaultTableModel {

		public CustomTableModel(Object[] colNames, int rowCount) {
			super(colNames, rowCount);
		}
		
		/*
		 * JTable uses this method to determine the default renderer/
		 * editor for each cell.  If we didn't implement this method,
		 * then the last column would contain text ("true"/"false"),
		 * rather than a check box.
		 */
		public Class getColumnClass(int c) {
			Object obj = getValueAt(0, c);

			// if null, default to class String			
			if ( obj == null ) 
				return String.class;
				 
			// otherwise return the class name of the object				 
			else 
				return obj.getClass(); 
		}
	}


	// determine if input is a valid function
	protected boolean validFunction(String name, String desc) {
		// both name and desc are not empty, so it's valid
		if ( !Global.isEmptyLine(name) && !Global.isEmptyLine(desc) ) 
			return true; 

		// both name and desc are empty, so it's valid... cuz we'll just ignore it 
		// (can arise if user creates a new row in the table and then doesnt fill in any values)
		else if ( Global.isEmptyLine(name) && Global.isEmptyLine(desc) )
			return true;
		
		// otherwise only one of the values has been given, so we need the user 
		// to supply both of them 
		else 
			return false;  
	}

	// used by ConfDialog during the "Save" event 
	// ensures that all Grail+ functions have a name and a desc (mandatory fields)
	public boolean validation() {
		Vector vRows, colValues;
		Enumeration e;
		String name, desc;  
		
		// make sure all cells' edit status are disabled before validating the data
		if ( getFuncTable().isEditing() )
			this.setEditing(false);
		
		vRows = this.getTableData();
		e = vRows.elements();
		
		// iterate through all the rows, ensuring valid values entered
		for (int i=0; i < vRows.size(); i++) {
			colValues = (Vector) e.nextElement();
			name = (String) colValues.elementAt(Global.FIRST_COL);
			desc  =  (String) colValues.elementAt(Global.SECOND_COL);
			
			// if not valid function, highlight offending row in the table
			if ( !validFunction(name, desc) ) {
				getFuncTable().setRowSelectionAllowed(true);
				getFuncTable().setRowSelectionInterval(i, i);
				
				// unsuccessful validation
				return false;
			}				
		}
		
		// otherwise, if we get here, it's a successful validation
		return true; 
	}

	
  
  // GETTERS & SETTERS //
  
  // change the selected cell in the table
  protected void setSelectedCell(int row, int col) {
	this.getFuncTable().changeSelection(row, col, false, false);
	
	// apparently we need to request focus to see the change in cell selection 
	getFuncTable().requestFocusInWindow();
  }
  
  
  protected void setButtonsEqualSized() {
    Dimension addAfter  = getAddAfterButton().getPreferredSize();
    Dimension addBefore = getAddBeforeButton().getPreferredSize();
    Dimension delete    = getDeleteButton().getPreferredSize();

    // get the tallest dimensions
    int widest  = Math.max(addAfter.width,  Math.max(addBefore.width, delete.width) );
    int tallest = Math.max(addAfter.height, Math.max(addBefore.height, delete.height) );

    // make all the buttons the same size
    setStaticDimensions(widest, tallest, getAddAfterButton() );
    setStaticDimensions(widest, tallest, getAddBeforeButton() );
    setStaticDimensions(widest, tallest, getDeleteButton() );
  }

  
  protected void setStaticDimensions(int width, int height, JButton sourceButton) {
    sourceButton.setMinimumSize(new Dimension(width, height) );
    sourceButton.setPreferredSize(new Dimension(width, height) );
    sourceButton.setMaximumSize(new Dimension(width, height) );
  }

		


  public void setEditing(boolean value) {
    if ( value == false ) {
      TableCellEditor editor = getFuncTable().getCellEditor();
      editor.stopCellEditing();
    }
  }

  public boolean isEditing()                       { return getFuncTable().isEditing(); }
  public Vector getTableData()                     { return getTableModel().getDataVector(); }
  
  protected JButton getAddBeforeButton() { return this.addBeforeButton; }
  protected void setAddBeforeButton(JButton addBeforeButton) { this.addBeforeButton = addBeforeButton; }

  protected JButton getAddAfterButton()  { return this.addAfterButton; }
  protected void setAddAfterButton(JButton addAfterButton)   { this.addAfterButton = addAfterButton; }

  protected JButton getDeleteButton()    { return this.deleteButton; }
  protected void setDeleteButton(JButton deleteButton)       { this.deleteButton = deleteButton; }

  protected void setButtonPanel(JPanel buttonPanel) { this.buttonPanel = buttonPanel; }
  protected JPanel getButtonPanel() { return this.buttonPanel; }

  // PUBLIC //
  public void setModified(boolean isModified) { this.isModified = isModified; }
  public boolean isModified()  { return this.isModified; }

  protected void setTableModel(DefaultTableModel tableModel) { this.tableModel = tableModel; }
  protected DefaultTableModel getTableModel() { return this.tableModel; }

  protected void setFuncTable(JTable funcTable) { this.funcTable = funcTable; }
  protected JTable getFuncTable() { return this.funcTable; }

  protected void setTableScrPane(JScrollPane tableScrPane) { this.tableScrPane = tableScrPane; }
  protected JScrollPane getTableScrPane() { return this.tableScrPane; }

  public static void main(String[] arg) throws Exception {
    JFrame frame = new JFrame("test");

    ConfTablePanel pan = new ConfTablePanel(400, 400, null);

      //frame.getContentPane().setLayout(new BoxLayout(frame.getContentPane(), BoxLayout.PAGE_AXIS) );
    frame.setContentPane(pan);

    frame.setSize(600, 600);
    frame.show();
  }
}
