//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*; 

/**
 * 
 * <p>
 * 
 * @author Aaron Cooper
 * @version 1.0 4-Jun-2005
 * 
 * The options listed for configuring the Tools in the ConfDialog.
 */

public class ConfToolsPanel extends AbstractPanel {

	private JPanel topPanel, bottomPanel;
	
	// arrays will hold references to the textfields
	private JTextField[] arrName, arrDesc, arrCmd; 
	
	// used in validation routine 
	private JTextField offendingElem; 

	public ConfToolsPanel(Function[] tools) {
		createComponents();
		setupComponents(tools);
		addComponents();
	}

	protected void createComponents() {
		createArrays();
		createTopPanel();
		createBottomPanel(); 
	}
	
	protected void setupComponents() {} // required abstract method
	
	protected void setupComponents(Function[] tools) {
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

		getBottomPanel().setBorder(BorderFactory.createCompoundBorder(
		  BorderFactory.createTitledBorder("Converters"),
		  BorderFactory.createEmptyBorder(5, 5, 5, 5) )
		);
		
		// hide the "empty" label (just used for placement purposes)
		getTopPanel().getComponent(0).setVisible(false);
		getBottomPanel().getComponent(0).setVisible(false);
		
		// used by validation for highlighting elements
		setupMouseListeners(); 
		
		// assign the textfield values based on the tools array		
		setupInitValues(tools); 
	}

	protected void createArrays() {
		arrName = new JTextField[Global.NUM_TOOLS];
		arrDesc  = new JTextField[Global.NUM_TOOLS];
		arrCmd  = new JTextField[Global.NUM_TOOLS];
	}

	protected void createTopPanel() {
		JTextField field;
		int i = 0; 
		
		setTopPanel(new JPanel() );
		getTopPanel().setLayout(new SpringLayout() );

		getTopPanel().add(new JLabel("Empty") );
		getTopPanel().add(new JLabel("Name") );		
		getTopPanel().add(new JLabel("Description") );
		getTopPanel().add(new JLabel("Command") );
		
		getTopPanel().add(new JLabel("Viewer:") );
		field 			 = new JTextField(15); 
		arrName[i] = field; 
		getTopPanel().add(field);
		field 			 = new JTextField(15); 
		arrDesc[i] = field; 
		getTopPanel().add(field);
		field 			 = new JTextField(30); 
		arrCmd[i] = field; 
		getTopPanel().add(field);

		i++;

		getTopPanel().add(new JLabel("Editor:") );
		field 			 = new JTextField(15); 
		arrName[i] = field; 
		getTopPanel().add(field);
		field 			 = new JTextField(15); 
		arrDesc[i] = field; 
		getTopPanel().add(field);
		field 			 = new JTextField(30); 
		arrCmd[i] = field; 
		getTopPanel().add(field);

		SpringUtilities.makeCompactGrid(getTopPanel(),
										3, 4, //rows, cols
										6, 6,        //initX, initY
										10, 10);       //xPad, yPad
	}
	
	protected void createBottomPanel() {
		JTextField field;
		int i = 2; // (top panel adds elements to indices 0, 1) 

		setBottomPanel(new JPanel() ); 		
		getBottomPanel().setLayout(new SpringLayout() );

		getBottomPanel().add(new JLabel("Empty") );
		getBottomPanel().add(new JLabel("Name") );		
		getBottomPanel().add(new JLabel("Description") );
		getBottomPanel().add(new JLabel("Command") );

		getBottomPanel().add(new JLabel("Finite Machine to Editor:") );
		field 			 = new JTextField(15); 
		arrName[i] = field; 
		getBottomPanel().add(field);
		field 			 = new JTextField(15); 
		arrDesc[i] = field; 
		getBottomPanel().add(field);
		field 			 = new JTextField(30); 
		arrCmd[i] = field; 
		getBottomPanel().add(field);

			/* Right now not applicable... but these conversion tools could be used later 
			i++;
	
			getBottomPanel().add(new JLabel("Reg Exp to Editor:") );
			field 			 = new JTextField(15); 
			arrName[i] = field; 
			getBottomPanel().add(field);
			field 			 = new JTextField(15); 
			arrDesc[i] = field; 
			getBottomPanel().add(field);
			field 			 = new JTextField(30); 
			arrCmd[i] = field; 
			getBottomPanel().add(field);
	
			i++;
	
			getBottomPanel().add(new JLabel("Finite Lang to Editor:") );
			field 			 = new JTextField(15); 
			arrName[i] = field; 
			getBottomPanel().add(field);
			field 			 = new JTextField(15); 
			arrDesc[i] = field; 
			getBottomPanel().add(field);
			field 			 = new JTextField(30); 
			arrCmd[i] = field; 
			getBottomPanel().add(field);
			*/

		SpringUtilities.makeCompactGrid(getBottomPanel(),
										2, 4, //rows, cols
										6, 6,        //initX, initY
										10, 10);       //xPad, yPad
	}
	
	protected void addComponents() {
		this.add(getTopPanel() );
		this.add(Box.createRigidArea(new Dimension(0, 50) ));
		this.add(getBottomPanel() );										
	}
	
	// setup the mouse listneres for dealing with error highlighting 
	protected void setupMouseListeners() {
		// create mouse handler 		
		MouseAdapter m = new MouseAdapter() {
			// when error messages occur, the textfield can be highlighted with an Error Color 
			// turn the background color to its original color once the user clicks in the textfield 
			public void mouseClicked(MouseEvent e) {
				JTextField source = (JTextField) e.getSource();
			
				// reset background color to its original color
				if ( source.getBackground() == Global.ERROR_HIGHLIGHT_COLOR )
					source.setBackground(Color.WHITE);  
			}
		}; 			
		
		// register mouse listeners with the textfields 
		// note the "desc" textfields are never highlighted, so they are omitted here 
		for (int i=0; i < Global.NUM_TOOLS; i++) {
			arrName[i].addMouseListener(m);
			arrCmd[i].addMouseListener(m);			
		}
	}
	
	
	// determine if valid input has been entered for the given three textfields
	// name and command are mandatory, while desc is not
	protected boolean validTool(JTextField nameField, JTextField descField, JTextField cmdField) {
		String name, cmd;
		
		name = nameField.getText();
		cmd = cmdField.getText(); 
		
		// both name and cmd are not empty, so it's valid
		if ( !Global.isEmptyLine(name) && !Global.isEmptyLine(cmd) ) 
			return true; 

		// name is empty, but cmd is not
		else if ( Global.isEmptyLine(name) && !Global.isEmptyLine(cmd) ) {
			offendingElem = nameField;
			return false;
		}

		// cmd is empty, name is not
		else if ( !Global.isEmptyLine(name) && Global.isEmptyLine(cmd) ) {
			offendingElem = cmdField;
			return false;
		}
		
		// both empty
		else if ( Global.isEmptyLine(name) && Global.isEmptyLine(cmd) ) {
			// desc field is not empty, so tool requires name and command values
			if ( !Global.isEmptyLine(descField.getText() )) {
				offendingElem = descField;
				return false; 
			}
			
			// otherwise all are empty, so we'll ignore it
			else			
				return true; 
		}

		// this case shouldn't happen
		else 
			return true; 
	}
	

	// validate the input (used by ConfDialog)
	public boolean validation() {
		// iterate through all the textfields to check for complete entries
		for (int i=0; i < Global.NUM_TOOLS; i++) {
			// if data is missing, it's an unsuccessful validation
			if ( !validTool(arrName[i], arrDesc[i], arrCmd[i]) ) {
				// if desc is not empty, while the other two are, highlight the two				
				if ( offendingElem == arrDesc[i] ) {
					arrName[i].setBackground(Global.ERROR_HIGHLIGHT_COLOR);
					arrCmd[i].setBackground(Global.ERROR_HIGHLIGHT_COLOR);					
				}
				
				// otherwise highlight the offending element
				else {
					offendingElem.setBackground(Global.ERROR_HIGHLIGHT_COLOR);
				}
				
				// unsuccessful validation				
				return false;
			}
		}
		
		// otherwise successful validation		
		return true;
	}
	
	
	
	// returns a Tool object, whose info comes from the textfields in the source panel
	private Function createTool(int index) {
		String name, desc, cmd;
	
		// get the data
		name = arrName[index].getText();
		desc = arrDesc[index].getText();
		cmd = arrCmd[index].getText();

		// trim the strings, or use the empty string if needed
		if ( name == null ) name = Global.EMPTY_STRING; else name.trim();
		if ( desc == null )   desc  = Global.EMPTY_STRING; else desc.trim();
		if ( cmd == null )   cmd   = Global.EMPTY_STRING; else cmd.trim();
		
		// create a Tool object and return it
		// don't set the path to the tool... will be set during the Save process 
		return new Function(name, desc, cmd, null); 
	}
	
	
	public Function[] getTools() {
		Function[] tools = new Function[Global.NUM_TOOLS];

		for (int i=0; i < Global.NUM_TOOLS; i++) {
				tools[i] = createTool(i);
		}

		// return the list of tools		
		return tools;
	}
	
	// assign the textfield values based on the tools array
	private void setupInitValues(Function[] tools) {
		Function f; 
		 
		for (int i=0; i < Global.NUM_TOOLS; i++) {
			f = tools[i]; 
			
			// if tool is not null, assign the values
			if ( f != null ) {
				arrName[i].setText(f.getName() );
				arrDesc[i].setText(f.getDesc() );
				arrCmd[i].setText(f.getCmd() );
			}
		}
	}
	

	
	// GETTERS and SETTERS
	protected void setTopPanel(JPanel topPanel) { this.topPanel = topPanel; }
	protected JPanel getTopPanel() { return this.topPanel; }
	
	protected void setBottomPanel(JPanel bottomPanel) { this.bottomPanel = bottomPanel; }
	protected JPanel getBottomPanel() { return this.bottomPanel; }

	public static void main(String[] args) {
		JFrame test = new JFrame();
		test.setSize(800, 800);
		
		ConfToolsPanel pan = new ConfToolsPanel(null); 
		test.getContentPane().add(pan);
		
		test.show(); 
	}
	
}
