//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.*;
import java.awt.event.*;

/**
* The dialog window that allows the user to edit a finite state machine 
* using an external program (such as Xfig).  The user chooses "Edit as Diagram" from the Tools menu.
* <p>
* 
*
* @author Aaron Cooper
* @version 1.0 May 27, 2005
*/


public class EditDiagDialog extends DiagDialog {
	 // used to obtain the viewer/editor
	 private ConfFile confFile;
	 
	 // debugging variable
	 private boolean D = true;
	 
	 public EditDiagDialog(JFrame parent, RightPanel rightPanel, ConfFile confFile) {
		 super(parent, rightPanel, "Edit");
		 
		setConfFile(confFile);
		
		setTitle("Edit Finite Machine as Diagram...");
	 }
	 
	public void actionPerformed(ActionEvent ae) {
		// let the superclass handle the bulk 
		super.actionPerformed(ae);

		// now execute the viewer
		// getInputFile() -- temp file is obtained via superclass' actionPerformed
		if ( getInputFile() != null ) {
			// now display the editor
			showEditor(getInputFile() );
		
			// ensure we only execute the command if inputFile has been re-assigned			
			setInputFile(null);
		}			 	
	}		 
	
	
	// converts the inputfile into the editor's file type
	protected String convert(String inputFile) {
		boolean execCmd; 
		String outFile; 
		
		// if no convertor exists, we'll use the inputfile as is
		if ( getFmToEditor() == null )
			return inputFile; 
		
		// name of the output file for the conversion program
		outFile = inputFile + "conv";
	
		getFmToEditor().addInFile(inputFile);
		getFmToEditor().setOutFile(outFile); 

			if (D) System.out.println("EditDiagDialog: convert(): going to try conversion");

		// try conversion		
		try {
			execCmd = getFmToEditor().execNoErrorHandling();
		}
		// if a rare exception occurs, we'll set boolean value to false
		catch (Exception e) {
			execCmd = false;
		}
		
		// if command failed, we'll try running the editor on the inputfile
		if ( !execCmd )
			return inputFile;

		// otherwise we'll run the editor on the converted file (i.e. output of the conversion program)
		else
			return outFile;  			
	}
	
	// displays the editor program, using inputFile as its input
	protected void showEditor(String inputFile) {
			if (D) System.out.println("showEditor(): inputFile coming in is..." + inputFile);
			
		// convert the file to the editor's file type 	
		inputFile = convert(inputFile);
		
			if (D) System.out.println("showEditor(): inputFile to editor is..." + inputFile);

		// setup the input file
		getEditor().addInFile(inputFile);
		
		// execute the command
		getEditor().execAsThread(); 
	}
	 
	 
	// GETTERS and SETTERS
	protected ConfFile getConfFile() { return this.confFile; }
	protected void setConfFile(ConfFile confFile) { this.confFile = confFile; }

	protected Function getEditor() 		{ return getConfFile().getEditor(); }
	protected Function getFmToEditor() { return getConfFile().getFmToEditor(); }

}
