//Author 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*; 
import javax.swing.*;
import java.io.*;

/**
* The dialog window that allows the user to view a finite state machine 
* using an external program (such as Xfig).  The user chooses "View as Diagram" from the Tools menu.
* <p>
* 
*
* @author Aaron Cooper
* @version 1.0 May 27, 2005
* Project JGrail
*/

public class ViewDiagDialog extends DiagDialog {
	// used to obtain the viewer external program	
	private ConfFile confFile;  

	public ViewDiagDialog(JFrame parent, RightPanel rightPanel, ConfFile confFile) {
		super(parent, rightPanel, "View");
		
		setConfFile(confFile);
		
		setTitle("View Finite Machine as Diagram...");
	}
	
	public void actionPerformed(ActionEvent ae) {
		// let the superclass handle the bulk 
		super.actionPerformed(ae);

		// now execute the viewer
		// getInputFile() -- temp file is obtained via superclass' actionPerformed
		if ( getInputFile() != null ) {
			showViewer(getInputFile() );
		
			// ensure we only execute the command if inputFile has been re-assigned			
			setInputFile(null);
		}			 	
	}		 
	
	
	// displays the viewer program, using inputFile as its input
	protected void showViewer(String inputFile) {
		System.out.println("inputFile=" + inputFile);
		getViewer().addInFile(inputFile);
			
		// execute the command
		getViewer().execAsThread(); 
	}

	protected ConfFile getConfFile() { return this.confFile; }
	protected void setConfFile(ConfFile confFile) { this.confFile = confFile; }
	
	protected Function getViewer() 		{return getConfFile().getViewer(); }

	public static void main(String[] args) {
		JFrame frmTest = new JFrame();
		frmTest.setSize(800, 600);

	   ViewDiagDialog dd = new ViewDiagDialog(frmTest, null, null);
	   dd.show();
	}
}
