//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.Color;
import java.io.File;

/** 
* Stores global variables and functions that can be used by many classes.  
* An attempt to provide some convenience and consistency.
*
* @author Aaron Cooper
* @version 1.0 May-June 2004
*/ 

public class Global {

  // constants for parsing
  public static final char   UNIX_SLASH    = '/';
  public static final char   COMMENT_CHAR  = '#';
  public static final char   SPACE_CHAR    = ' ';

  public static final String EQUAL_SIGN    = "=";
  public static final String UNIX_PIPE     = "|";
  public static final String RE_DIRECT     = ">";

  public static final String EMPTY_STRING  = "";
  public static final String EMPTY_TEXT    = " ";

  public static final String DFLT_TXT_FLD_FILE_PATH = "<enter location>";
  public static final String SAVE_FILE_MSG = "Successfully saved the file: ";
  
  public static final int FIRST_COL  	 = 0;
  public static final int SECOND_COL = 1;
  public static final int THIRD_COL 	 = 2;
  

  public static final int NOT_FOUND = -1;

  public static final Color STATUS_ERROR_MSG_COLOR = Color.RED;
  public static final Color STATUS_SAVE_MSG_COLOR  = Color.BLUE;
  
  /* Name of Files */ 
  /** location where the installation places the files */
  public static final String JGRAIL_LOCATION = "/usr/local/jgrail";
  
  /** User's home directory (e.g. /home/aaron) */ 
  public static final String USER_HOME_LOCATION = System.getProperty("user.home");


  // used for the background color of the JGrail panels 
  public static final Color PEACH = new Color(255, 255, 204); 
	
  // used to highlight a GUI widget when an error occurs, grabbing the user's attention
  public static final Color ERROR_HIGHLIGHT_COLOR = Color.orange;
  
  
  // used by looksLikeFM() below 
  public static final String START_STATE = "START";
  public static final String FINAL_STATE  = "FINAL";
  
  // maximum number of tools that should be found under the Tools section in the conf file
  public static final int NUM_TOOLS = 3; 
  
	
	/** 
	 * Adds a slash to a directory path, if it doesn't already have one
	 *
	 * @param path the path to possibly add the slash to
	 * @return the path with a slash at its tail end
	 */ 
	 public static String addSlash(String path) {
		if ( path != null ) {
			path.trim();
			
			// if the PATH doesn't end with a "/", ... then we add one
			if ( path.length() > 0 && path.charAt(path.length() - 1) != Global.UNIX_SLASH )
			  path = path + Global.UNIX_SLASH;
		}
		
		return path; 
	 }
	 
	/** 
	* Determines if the given argument is an empty line 
	* 
	* @param line the line to check for emptiness
	* @return boolean value, representing whether or not the given argument is empty
	*/ 
	public static boolean isEmptyLine(String line) {
	  if ( line == null )
		return true;

	  return line.length() == 0;
	}

	// determines if a backup version of the configuration file exists
	public static boolean backupFileExists() {
		File f = new File(ConfFile.CONF_LOCATION + Global.UNIX_SLASH + ConfFile.BAK_CONF_FILE);
		
		return f.exists();
	}
	 
	// instantiates and shows the bug error message	 
	public static void showBugErrorDialog(Exception e) {
		BugErrorDialog dlg = new BugErrorDialog(e);
		dlg.show(); 	
	}
	

	// determines if String looks like a finite machine (used by RightClickMenu and View/Edit Dialogs)
	// very simple check -- just looks for one of each: (START) state and (FINAL) state
	public static boolean looksLikeFM(String text) {
		if ( text == null || text.length() == 0 )
			return false;
		
		// split the string by newlines
		String[] split = text.split("\n");
		
		boolean foundStart = false; 
		
		if ( split.length == 0 )
			return false;

		// look for at least one start state
		for (int i=0; i < split.length; i++) {  
			System.out.println("split[" + i + "] = " + split[i] );
			
			if ( split[i].indexOf(START_STATE) != NOT_FOUND )
				foundStart = true;
		}
			
		// if not found, then doesnt look like an FM			
		if ( !foundStart )
			return false;

		// now look for at least one final state
		for (int i=0; i < split.length; i++) {  
			// if we have found at least one start and final state, it looks like an FM
			if ( split[i].indexOf(FINAL_STATE) != NOT_FOUND )
				return true;
		}
					
		// if we get here, then it doesnt look like an FM		
		return false;
	}
	

	 
}
