//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.io.*;
import java.nio.channels.*;

/**
 * 
 * Creates the .jgrail directory under the user's home directory and copies over the default
 * configuration file.
 * <p>
 * 
 * @author Aaron Cooper
 * @version 1.0 2-Jul-2005
 */

public class Setup {
	// name of temporary directory that will be created under the setup dir
	private static final String TEMP_DIR = ConfFile.CONF_LOCATION + "/tmp";

	// dir where default files are located
	private static final String SOURCE_DIR = Global.JGRAIL_LOCATION + "/dflt";
	
	// name of the help file
	private static final String HELP_FILE = "/help.html";

	// debugging variable
	private boolean D = false; 

	// creates the configuration directory (eg /home/aaron/.jgrail) and copies over the default conf files	
	public boolean create() throws Exception {
		File setupDir   = new File(ConfFile.CONF_LOCATION); 
		File sourceDir = new File(SOURCE_DIR);			
	
		File helpFileIn, helpFileOut, tempDir; 
		
		// only copy over the files if the setup dir doesn't already exist
		if ( !setupDir.exists() ) {
			
			// create the setup dir
			System.out.println("checking setupDir.mkdir() ln 48");
			setupDir.mkdir(); 					

			if (D) {System.out.println("helpFileIn refers to ... " + SOURCE_DIR + HELP_FILE);} 
			if (D) {System.out.println("helpFileOut refers to ..." + ConfFile.CONF_LOCATION + HELP_FILE);}

			// create the help file (source and destination)
			helpFileIn    = new File(SOURCE_DIR + HELP_FILE);
			helpFileOut = new File(ConfFile.CONF_LOCATION + HELP_FILE);

			// copy the help file over to the setup dir			
			copyFile(helpFileIn, helpFileOut);

			// create the temporary directory 
			tempDir  = new File(TEMP_DIR);
			tempDir.mkdir(); 
		
			// now copy over the default configuration file
			copyDfltConfFile();
			
			// returning true to signifiy that the directory needed to be created
			return true;  			
		}
		
		// if we get here, that means the directory didnt need to be created 
		return false;
	}
	
	// modified from: http://www.rgagnon.com/javadetails/java-0064.html
	private void copyFile(File in, File out) throws Exception {
		 FileChannel sourceChannel 		= new FileInputStream(in).getChannel();
		 FileChannel destinationChannel =  new FileOutputStream(out).getChannel();

		 try {
			// transfer all the bytes to the destination file
			// http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=5056395
			// transferFrom seems to work (as opposed to transferTo).. transferTo didnt work on Redhat Fedora 3
			destinationChannel.transferFrom(sourceChannel, 0, sourceChannel.size() );
		 }

		 // AC Aug 25, 2005
		 catch (Exception e) {
			System.out.println("message = " + e.getMessage() );     
			System.out.println("toString = " + e.toString() );     
			throw e; 
		 }	

		 finally { 
		 	// close the files
		 	sourceChannel.close();
		 	destinationChannel.close();
		 }
	}
		

	// copies over the default configuration file from the defaults folder
	public void copyDfltConfFile() throws Exception {
		File confIn, confOut;
		
		// create the files
		confIn	 = new File(SOURCE_DIR + Global.UNIX_SLASH + ConfFile.CONF_FILE);  
		confOut = new File(ConfFile.CONF_FULL_PATH); 
		

		// now copy over the default configuration file
		copyFile(confIn, confOut); 
	}
	
	public void restoreBackup() throws Exception {
		File confIn, confOut; 
		
		// create the files
		confIn	 = new File(ConfFile.CONF_LOCATION + Global.UNIX_SLASH + ConfFile.BAK_CONF_FILE); 
		confOut = new File(ConfFile.CONF_FULL_PATH); 

		// now copy the backup conf file
		copyFile(confIn, confOut); 
	}
	
	public void print() {
		System.out.println(SOURCE_DIR);

		System.out.println(HELP_FILE);
		System.out.println(TEMP_DIR);
	}

	public static void main(String[] args) throws Exception {
		Setup s = new Setup(); 
		s.create();
	}
}
