//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import javax.swing.*;
import java.io.*;

/**
 * IOPanel is an abstract superclass for OutputPanel and InputPanel.  It contains a common 
 * writeTextToFile method.  It is not to be instantiated.
 * 
 * @author Aaron Cooper
 * @version 1.0 May 28, 2005
 */

abstract public class IOPanel extends AbstractPanel {

	// strings that are used in a Finite Machine's representation
	protected static final String FM_START = "(START)";
	protected static final String FM_FINAL  = "(FINAL)";

	private JScrollPane scrollPane; 
	private String tempFilePath; 

	// inherited abstract methods from AbstractPanel
	abstract protected void createComponents();
	abstract protected void setupComponents();
	abstract protected void addComponents();

	// write the textarea's data to a temporary file 
	// (computations are performed with input coming from a file)
	public boolean writeTextToFile() {
		  JTextArea textArea = getTextArea();
		  
		  // if no text has been entered, then don't bother writing to file
		  if ( textArea.getText() == null )
		  	return false;  
	
		  try {
			// try writing text to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(tempFilePath) );
			textArea.write(writer);

			// if we get here, then file writing was successful
			return true; 
		  }
		  
		  catch (Exception e) {
			// handle exceptions
			System.out.println("From IOPanel:writeTextToFile(): Exception when trying to write to file: " + tempFilePath);
			
			// unsuccessful file writing
			return false;
		  }
	}
	
	
	// determines if the input text is a Finite Machine
	// useful to prevent user from running Grail+ commands with inproper input
	public boolean containsFM() {
		String input = this.getTextArea().getText(); 
		
		// no text means it isn't an FM
		if ( input == null )
			return false; 
			
		// if the input contains the START and FINAL tags that represent an FM, it must be an FM			
		if ( input.indexOf(FM_START) != Global.NOT_FOUND && input.indexOf(FM_FINAL) != Global.NOT_FOUND )
			return true; 
		
		// otherwise, if we get here, it isn't an FM		
		return false; 
	}
	
	// GETTERS and SETTERS
	protected void setTempFilePath(String tempFileName) { 
		this.tempFilePath = ConfFile.TMP_LOCATION + Global.UNIX_SLASH + tempFileName; 
	}
	public String getTempFilePath() { return tempFilePath; }
	
	protected JScrollPane getScrPane() { return scrollPane; }
	protected void setScrPane(JScrollPane scrollPane) { this.scrollPane = scrollPane; }
	
	protected JTextArea getTextArea() { return (JTextArea) getScrPane().getViewport().getView(); }
}
