//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*; 

import java.io.*; 

/** 
* Represents the panel in which the user inputs the data required by the Grail+ functions.  
* It is located on the right side of the main JGrail screen, above the Output panel. 
* 
* @author Aaron Cooper
* @version 1.0 May-June 2004
*/ 

class InputPanel extends IOPanel {
  /** Pixel-width of the empty border that surrounds the panel */ 
	protected static final int EMPTY_BDR_SIZE = 10;
  /** Pixel-width of an empty component that is placed between the GUI widgets */ 
  protected static final int EMPTY_CMP_SIZE = 5;
  /** Title of the browse button */ 
  protected static final String BTN_TITLE   = "Browse";

	private FileNamePanel fileNamePanel; 

	public InputPanel(String title, String tempFileName, JGrailGui parentFrame) {
		setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(title),
			BorderFactory.createEmptyBorder(EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE) )); 
		setLayout(new GridBagLayout() ); 
		
		createComponents(parentFrame);
		setupComponents();
		addComponents(); 
		
		setTempFilePath(tempFileName);			
	}

  	//// GETTERS /////
	public JTextArea getInputTextArea() { return super.getTextArea(); } // PUBLIC //

  		
	protected void setInputScrPane(JScrollPane inputScrPane) { super.setScrPane(inputScrPane); }
	protected JScrollPane getInputScrPane() { return super.getScrPane(); }

	protected void setFileNamePanel(FileNamePanel fileNamePanel) { this.fileNamePanel = fileNamePanel; }
	protected FileNamePanel getFileNamePanel() { return this.fileNamePanel; }
	

  // required abstract method
  protected void createComponents() {}
  
  /** 
  * Creates the GUI widgets for this Panel 
  */  	
	protected void createComponents(JGrailGui parentFrame) {
    	//setFileNameTxtFld(new JTextField(Global.DFLT_TXT_FLD_FILE_PATH) );
    	//setBrowseButton(new JButton(BTN_TITLE) );

    	setInputScrPane(new JScrollPane(new JTextArea() ));
    	setFileNamePanel(new FileNamePanel(parentFrame.getStatusBar() ));
	}

  /** 
  * Adds the GUI widgets to this Panel 
  */  	
  protected void addComponents() {
    GridBagConstraints c = new GridBagConstraints();

    c.fill  = GridBagConstraints.BOTH;

    c.gridx = 0;
    c.gridy = 0;

    // expand horizontally
    c.weightx = 1.0;
    c.weighty = 0.0;

    c.insets = new Insets(0, 0, 5, 0);
    add(getFileNamePanel(), c);

    c.gridx = 0;
    c.gridy = 1;

    // expand both
    c.weightx = 1.0;
    c.weighty = 1.0;

    c.insets = new Insets(5, 0, 0, 0);
    add(getInputScrPane(), c);
  }

	
  /** 
  * Required abstract method
  */  	
  protected void setupComponents() {}
}
