//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*;  
import javax.swing.*; 

/**
 * Output panel that is found on the JGrail main screen.  This is where the computations are placed. 
 * 
 * 
 * @author Aaron Cooper
 * @version 1.0 May 2005
 */

class OutputPanel extends IOPanel {
  protected static final int EMPTY_BDR_SIZE = 10;
  protected static final int EMPTY_CMP_SIZE = 5;
  
  protected static final String PANEL_TITLE = "Output";
  protected static final String BTN_TITLE   = "Compute";

  private JButton     computeButton; 

  public OutputPanel(String tempFileName) {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS) ); 

	setBorder(BorderFactory.createCompoundBorder(
	  BorderFactory.createTitledBorder(PANEL_TITLE),
	  BorderFactory.createEmptyBorder(EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE, EMPTY_BDR_SIZE) )
	); 

	createComponents();
	setupComponents();
	addComponents();

	setTempFilePath(tempFileName);  	
  }	  	

  // GETTERS //
  public JTextArea getOutputTextArea() { return super.getTextArea(); } // PUBLIC //

  protected void setOutputScrPane(JScrollPane outputScrPane) { super.setScrPane(outputScrPane); }
  protected JScrollPane getOutputScrPane() { return super.getScrPane(); }


  protected void setComputeButton(JButton computeButton) { this.computeButton = computeButton; }
  protected JButton getComputeButton() { return this.computeButton; }

  public void setComputeButtonHandler(ActionListener listener) {
    getComputeButton().addActionListener(listener);
  }

  protected void createComponents() {
    setOutputScrPane(new JScrollPane(new JTextArea() ));
    setComputeButton(new JButton(BTN_TITLE) );
  }

  protected void setupComponents() {
    getComputeButton().setAlignmentX(CENTER_ALIGNMENT);
  }


  protected void addComponents() {
    add(getOutputScrPane() );
    // empty space between textarea and button    
    add(Box.createRigidArea(new Dimension(0, EMPTY_CMP_SIZE) ));
    add(getComputeButton() );
  }
}
