//Copyright 2005 Aaron Cooper
/*
This file is part of JGrail.
JGrail is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

JGrail is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with JGrail; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
*/

import java.awt.*;
import java.awt.event.*;

import javax.swing.*; 

/**
 * Right-panel contains the InputPanels and OutputPanel.  That is, the right-hand side of the JGrail main screen.
 * 
 * @author Aaron Cooper
 * @version 1.0 May 2005
 */

public class RightPanel extends AbstractPanel implements ActionListener {
  protected static final String FIRST_INPUT_PANEL_TITLE  = "First Input";
  protected static final String SECOND_INPUT_PANEL_TITLE = "Second Input";

  protected static final String FIRST_CHK_BOX_TITLE      = "Hide First";
  protected static final String SECOND_CHK_BOX_TITLE     = "Hide Second";
  protected static final String OUTPUT_CHK_BOX_TITLE     = "Hide Output";    

  private OutputPanel outputPanel;
  private InputPanel  firstInputPanel, secondInputPanel; 
  private JPanel      hideChkBoxPanel;
  private JCheckBox   hideFirstChkBox, hideSecondChkBox, hideOutputChkBox; 

  
  // tempFileDir needed by IOPanels for writeTextToFile method
  public RightPanel(String tempFileDir, JGrailGui parentFrame) {
  	createComponents(tempFileDir, parentFrame);
  	setupComponents();
  	addComponents();
  }

  // register event handling
  protected void setupComponents() {
    getHideFirstChkBox().addActionListener(this);
    getHideSecondChkBox().addActionListener(this);
    getHideOutputChkBox().addActionListener(this);
  }

  //// GETTERS /////
  public InputPanel  getFirstInputPanel()                           			 { return this.firstInputPanel; }
  protected void setFirstInputPanel(InputPanel firstInputPanel)      { this.firstInputPanel    = firstInputPanel; }
  
  public InputPanel  getSecondInputPanel()                           { return this.secondInputPanel;}
  protected void setSecondInputPanel(InputPanel secondInputPanel)    { this.secondInputPanel   = secondInputPanel; }
  
  public OutputPanel getOutputPanel()                                { return this.outputPanel; }
  protected void setOutputPanel(OutputPanel outputPanel)             { this.outputPanel        = outputPanel; }

  protected JPanel getHideChkBoxPanel()                            { return this.hideChkBoxPanel; }
  protected void setHideChkBoxPanel(JPanel hideChkBoxPanel)        { this.hideChkBoxPanel  = hideChkBoxPanel; }

  
  public JCheckBox getHideFirstChkBox()                            { return this.hideFirstChkBox; }
  protected void setHideFirstChkBox(JCheckBox hideFirstChkBox)     { this.hideFirstChkBox  = hideFirstChkBox; }

  public JCheckBox getHideSecondChkBox()                           { return this.hideSecondChkBox; }
  protected void setHideSecondChkBox(JCheckBox hideSecondChkBox)   { this.hideSecondChkBox = hideSecondChkBox; }

  public JCheckBox getHideOutputChkBox()                           { return this.hideOutputChkBox; }
  protected void setHideOutputChkBox(JCheckBox hideOutputChkBox)   { this.hideOutputChkBox = hideOutputChkBox; }

  public void setComputeButtonHandler(ActionListener listener) { getOutputPanel().setComputeButtonHandler(listener); }


  // required abstract method from AbstractPanel	
  protected void createComponents() {}

  // tempFileDir needed by IOPanels for writeTextToFile method	   
  protected void createComponents(String tempFileDir, JGrailGui parentFrame) {
    // components added to right
    setOutputPanel(new OutputPanel("output") );
    setFirstInputPanel(new InputPanel(FIRST_INPUT_PANEL_TITLE, "input1", parentFrame ));
    setSecondInputPanel(new InputPanel(SECOND_INPUT_PANEL_TITLE, "input2", parentFrame ));

    // panel to hold checkboxes to hide the panels
    setHideChkBoxPanel(new JPanel() );

    setHideFirstChkBox(new JCheckBox(FIRST_CHK_BOX_TITLE) );
    setHideSecondChkBox(new JCheckBox(SECOND_CHK_BOX_TITLE) );
    setHideOutputChkBox(new JCheckBox(OUTPUT_CHK_BOX_TITLE) );

    getHideChkBoxPanel().setLayout(new BoxLayout(getHideChkBoxPanel(), BoxLayout.LINE_AXIS) );

    // add components, creating empty space between them
    getHideChkBoxPanel().add(getHideFirstChkBox() );
    getHideChkBoxPanel().add(Box.createRigidArea(new Dimension(10, 0)) );
    getHideChkBoxPanel().add(getHideSecondChkBox() );
    getHideChkBoxPanel().add(Box.createRigidArea(new Dimension(10, 0)) );
    getHideChkBoxPanel().add(getHideOutputChkBox() );
  }

  protected void addComponents() {
    GridBagConstraints c = new GridBagConstraints();
    setLayout(new GridBagLayout() );

    c.anchor = GridBagConstraints.FIRST_LINE_START;
    c.fill   = GridBagConstraints.BOTH;

    c.gridx = 0;
    c.gridy = 0;

    c.weightx = 1.0;
    c.weighty = 0.3;

    c.insets = new Insets(10, 0, 0, 5);

    // first input panel on the top left
    add(getFirstInputPanel(), c);

    c.gridx = 1;
    c.gridy = 0;

    c.insets = new Insets(10, 5, 0, 0);

    // second input panel on the top right
    add(getSecondInputPanel(), c);

    c.gridx = 0;
    c.gridy = 1;

    c.weightx = 1.0;
    c.weighty = 0.7;

    c.insets    = new Insets(10, 0, 0, 0);
    c.gridwidth = GridBagConstraints.REMAINDER;

    // output panel in the middle
    add(getOutputPanel(), c);

    c.gridx = 0;
    c.gridy = 2;

    c.weightx = 1.0;
    c.weighty = 0.0;

    c.fill   = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.CENTER;

    c.insets = new Insets(0, 0, 0, 0);

    // checkbox panel on the bottom
    add(getHideChkBoxPanel(), c);

    // debug
    //setBorder(BorderFactory.createLineBorder(Color.BLACK) );
  }

  public void actionPerformed(ActionEvent ae) {
    JCheckBox source = (JCheckBox) ae.getSource();

    if ( source == getHideFirstChkBox() )
      getFirstInputPanel().setVisible(!getFirstInputPanel().isVisible() );
    else if ( source == getHideSecondChkBox() )
      getSecondInputPanel().setVisible(!getSecondInputPanel().isVisible() );
    else
      getOutputPanel().setVisible(!getOutputPanel().isVisible() );
  }
  
}
